<?php
namespace codeneric\phmm\base\includes {
  require_once ($GLOBALS["HACKLIB_ROOT"]);
  use \codeneric\phmm\Utils;
  use \codeneric\phmm\Logger;
  use \codeneric\phmm\base\Watermarker;
  class Image {
    public static function get_image(
      $id,
      $use_minithumb = false,
      $query_args = array()
    ) {
      if (\get_post_type($id) !== "attachment") {
        return null;
      }
      $query_args[\hacklib_id("attach_id")] = (string) $id;
      $imagedata = \wp_get_attachment_metadata($id);
      if (!\hacklib_cast_as_boolean(is_array($imagedata))) {
        return null;
      }
      $image = \get_post($id);
      $meta = array(
        "caption" =>
          \hacklib_cast_as_boolean(\is_null($image))
            ? null
            : $image->post_excerpt
      );
      $uncropped_sizes = Utils::get_uncropped_image_sizes();
      $uncropped_sizes_names = \array_keys($uncropped_sizes);
      $sizes =
        \hacklib_cast_as_boolean(\array_key_exists("sizes", $imagedata))
          ? $imagedata[\hacklib_id("sizes")]
          : array();
      $mapped_sizes = array();
      $available_uncropped_sizes =
        \array_intersect($uncropped_sizes_names, \array_keys($sizes));
      $no_uncropped_sizes_availalbe =
        \count($available_uncropped_sizes) === 0;
      $site_url = \site_url();
      $watermark = false;
      if (\hacklib_cast_as_boolean(
            \array_key_exists("project_id", $query_args)
          )) {
        $project_id = (int) $query_args[\hacklib_id("project_id")];
        $pc = Project::get_configuration($project_id);
        if (\hacklib_cast_as_boolean($pc[\hacklib_id("watermark")])) {
          $watermark = true;
        }
      }
      foreach ($sizes as $size_name => $size) {
        $s = \wp_get_attachment_image_src($id, $size_name);
        if (\hacklib_cast_as_boolean(is_array($s)) &&
            (\hacklib_cast_as_boolean($no_uncropped_sizes_availalbe) ||
             \hacklib_cast_as_boolean(
               \in_array($size_name, $uncropped_sizes_names)
             ))) {
          $url = (string) \add_query_arg($query_args, $s[0]);
          if (\hacklib_cast_as_boolean($watermark)) {
            $url = self::get_watermark_url($s[0], $query_args);
          }
          $url = Utils::get_protocol_relative_url($url);
          $mapped_sizes[] = array(
            "url" => $url,
            "width" => (int) $size[\hacklib_id("width")],
            "height" => (int) $size[\hacklib_id("height")],
            "name" => (string) $size_name
          );
        }
      }
      $t2 = Utils::time();
      $filename = \basename(\get_attached_file($id, true));
      $mini_thumb_b64 =
        \hacklib_cast_as_boolean($use_minithumb)
          ? self::get_minithumb($id)
          : null;
      if (\count($mapped_sizes) === 0) {
        $width =
          \hacklib_cast_as_boolean(\array_key_exists("width", $imagedata))
            ? ((int) $imagedata[\hacklib_id("width")])
            : null;
        $height =
          \hacklib_cast_as_boolean(\array_key_exists("height", $imagedata))
            ? ((int) $imagedata[\hacklib_id("height")])
            : null;
        if (($width !== null) && ($height !== null)) {
          $url = \wp_get_attachment_url($id);
          $url = Utils::get_protocol_relative_url($url);
          $mapped_sizes[] = array(
            "url" => $url,
            "width" => $width,
            "height" => $height,
            "name" => "original"
          );
        }
      }
      $image = array(
        "sizes" => $mapped_sizes,
        "filename" => $filename,
        "id" => (int) $id,
        "meta" => $meta,
        "mini_thumb" => (string) $mini_thumb_b64,
        "error" => false
      );
      return $image;
    }
    public static function get_minithumb($id) {
      $imagedata = \wp_get_attachment_metadata($id);
      if (!\hacklib_cast_as_boolean(is_array($imagedata))) {
        return null;
      }
      $filename = false;
      $medium_exists = \codeneric\phmm\Utils::array_nested_key_exist(
        "sizes.medium.file",
        $imagedata
      );
      $thumb_exists = \codeneric\phmm\Utils::array_nested_key_exist(
        "sizes.thumbnail.file",
        $imagedata
      );
      if (\hacklib_cast_as_boolean($medium_exists) ||
          \hacklib_cast_as_boolean($thumb_exists)) {
        $size_name =
          \hacklib_cast_as_boolean($medium_exists) ? "medium" : "thumbnail";
        $filename =
          $imagedata[\hacklib_id("sizes")][$size_name][\hacklib_id("file")];
        $o_path = \get_attached_file($id, true);
        $filename = \dirname($o_path)."/".$filename;
        if (!\hacklib_cast_as_boolean(\file_exists($filename))) {
          return null;
        }
      }
      if ($filename === false) {
        return null;
      }
      list($width, $height, $image_type) = \getimagesize($filename);
      $b64_path = \dirname($filename)."/".$id.".b64";
      if (!\hacklib_cast_as_boolean(\file_exists($b64_path))) {
        $newwidth = \min(20, \intval(20 * ($width / $height)));
        $newheight = \min(20, \intval(20 * ($height / $width)));
        $minithumb = \imagecreatetruecolor($newwidth, $newheight);
        $image_type_str = null;
        $source = null;
        switch ($image_type) {
          case \IMAGETYPE_GIF:
            $source = \imagecreatefromgif($filename);
            break;
          case \IMAGETYPE_JPEG:
            $source = \imagecreatefromjpeg($filename);
            $image_type_str = "jpeg";
            break;
          case \IMAGETYPE_PNG:
            $source = \imagecreatefrompng($filename);
            $image_type_str = "png";
            break;
          default:
            break;
        }
        if (($source === null) || ($image_type_str === null)) {
          return null;
        }
        \imagecopyresized(
          $minithumb,
          $source,
          0,
          0,
          0,
          0,
          $newwidth,
          $newheight,
          $width,
          $height
        );
        \ob_start();
        \imagejpeg($minithumb);
        $img = \ob_get_clean();
        $b64 = "data:image/".$image_type_str.";base64,".\base64_encode($img);
        \imagedestroy($minithumb);
        \imagedestroy($source);
        $fp = \fopen($b64_path, "w");
        \fwrite($fp, $b64);
        \fclose($fp);
        return $b64;
      } else {
        $fp = \fopen($b64_path, "r");
        $res = \fread($fp, \filesize($b64_path));
        \fclose($fp);
        return $res;
      }
    }
    public static function delete($image_id) {
      \wp_delete_attachment($image_id, true);
      self::delete_watermarked_versions($image_id);
    }
    public static function get_original_image_url($id, $query_args = array()) {
      $query_args[\hacklib_id("attach_id")] = $id;
      $img_arr = \wp_get_attachment_image_src($id, "full");
      if ($img_arr === false) {
        return null;
      }
      \HH\invariant(
        \hacklib_cast_as_boolean(\is_array($img_arr)) &&
        (\count($img_arr) >= 4) &&
        \hacklib_cast_as_boolean(\is_string($img_arr[0])),
        "%s",
        new Error("Bad format of wp_get_attachment_image_src return.")
      );
      $url = (string) \add_query_arg($query_args, $img_arr[0]);
      $url = Utils::get_protocol_relative_url($url);
      return array("url" => $url);
    }
    private static function get_watermark_url($url, $query_args) {
      $url = self::unjetpackify_url($url);
      $file_path = self::transform_url_to_path($url);
      $uploaded_with_phmm_4_3_6_or_later =
        self::was_uploaded_with_phmm_4_3_6_or_later($url);
      $last_time_wm_settings_updated =
        self::get_last_time_wm_settings_updated();
      $settings = \codeneric\phmm\base\admin\Settings::getCurrentSettings();
      $wms = $settings[\hacklib_id("watermark")];
      $fast_image_load = $settings[\hacklib_id("fast_image_load")];
      if (!\hacklib_cast_as_boolean($fast_image_load)) {
        return self::get_legacy_watermark_url($url, $query_args);
      }
      $wm_args = array(
        "file" => $file_path,
        "output_to_browser" => false,
        "wms" => $wms
      );
      if (\hacklib_cast_as_boolean($uploaded_with_phmm_4_3_6_or_later)) {
        $watermarked_image_path =
          self::transform_to_watermarked_path($file_path);
        $res_url = self::transform_path_to_url($watermarked_image_path);
        if (\hacklib_cast_as_boolean(\file_exists($watermarked_image_path))) {
          $ft = \filemtime($watermarked_image_path);
          if ($ft > $last_time_wm_settings_updated) {
            $res_url = (string) \add_query_arg($query_args, $res_url);
            return $res_url;
          }
        }
        $wm_args[\hacklib_id("output_to_file")] = $watermarked_image_path;
        Watermarker::watermark_image($wm_args);
        $res_url = (string) \add_query_arg($query_args, $res_url);
        return $res_url;
      }
      $legacy_watermarked_image_path =
        self::transform_to_legacy_watermarked_path($file_path);
      $res_url = self::transform_path_to_url($legacy_watermarked_image_path);
      if (\hacklib_cast_as_boolean(
            \file_exists($legacy_watermarked_image_path)
          )) {
        $ft = \filemtime($legacy_watermarked_image_path);
        if ($ft > $last_time_wm_settings_updated) {
          $res_url = (string) \add_query_arg($query_args, $res_url);
          return $res_url;
        }
      }
      $wm_args[\hacklib_id("output_to_file")] =
        $legacy_watermarked_image_path;
      Watermarker::watermark_image($wm_args);
      $res_url = (string) \add_query_arg($query_args, $res_url);
      return $res_url;
    }
    private static function get_legacy_watermark_url($url, $query_args) {
      $site_url = \site_url();
      $matches = array();
      \preg_match("/.*\\/photography_management\\/(.*)/", $url, $matches);
      if (\count($matches) === 2) {
        $f = $matches[1];
        $new_query_args = $query_args;
        $new_query_args[\hacklib_id("f")] = $f;
        $new_query_args[\hacklib_id("codeneric_load_image")] = "1";
        return (string) \add_query_arg($new_query_args, $site_url);
      }
      return $url;
    }
    private static function was_uploaded_with_phmm_4_3_6_or_later($url) {
      $res = \preg_match("|.*/photography_management/.*/imgs/.*|", $url);
      return \hacklib_equals($res, 1);
    }
    private static function get_last_time_wm_settings_updated() {
      $upload_dir = \wp_upload_dir();
      $basedir = $upload_dir[\hacklib_id("basedir")];
      $wm_time_file = $basedir."/photography_management/wm-updated.txt";
      if (!\hacklib_cast_as_boolean(\file_exists($wm_time_file))) {
        return 0;
      }
      return \filemtime($wm_time_file);
    }
    private static function transform_url_to_path($url) {
      $upload_dir = \wp_upload_dir();
      $basedir = $upload_dir[\hacklib_id("basedir")];
      $baseurl = $upload_dir[\hacklib_id("baseurl")];
      $rel_url = Utils::get_protocol_relative_url($url);
      $rel_baseurl = Utils::get_protocol_relative_url($baseurl);
      $matches = array();
      \preg_match("|".$rel_baseurl."/(.*)|", $rel_url, $matches);
      if (\count($matches) === 2) {
        $f = $matches[1];
        return $basedir."/".$f;
      }
      \HH\invariant(
        false,
        "%s",
        new Error("Image URL to image path transformation failed: ".$url)
      );
    }
    private static function unjetpackify_url($url) {
      $url = \preg_replace("#//i[0-9]\\.wp\\.com/#", "//", $url);
      $url = \preg_replace("#\\?.*#", "", $url);
      return $url;
    }
    private static function transform_path_to_url($path) {
      $upload_dir = \wp_upload_dir();
      $basedir = $upload_dir[\hacklib_id("basedir")];
      $baseurl = $upload_dir[\hacklib_id("baseurl")];
      $matches = array();
      \preg_match("|".$basedir."/(.*)|", $path, $matches);
      if (\count($matches) === 2) {
        $f = $matches[1];
        return $baseurl."/".$f;
      }
      \HH\invariant(
        false,
        "%s",
        new Error("Image path to image URL transformation failed: ".$path)
      );
    }
    private static function transform_to_watermarked_path($f) {
      return \dirname(\dirname($f))."/watermarked/".\basename($f);
    }
    private static function transform_to_legacy_watermarked_path($f) {
      return \dirname($f)."/cc_phmm_wm_".\basename($f);
    }
    public static function delete_watermarked_versions($image_id) {
      $imagedata = \wp_get_attachment_metadata($image_id);
      if (!\hacklib_cast_as_boolean(is_array($imagedata))) {
        return;
      }
      $sizes =
        \hacklib_cast_as_boolean(\array_key_exists("sizes", $imagedata))
          ? $imagedata[\hacklib_id("sizes")]
          : array();
      foreach ($sizes as $size_name => $size) {
        $s = \wp_get_attachment_image_src($image_id, $size_name);
        if (\hacklib_cast_as_boolean(is_array($s))) {
          $url = $s[0];
          $path = self::transform_url_to_path($url);
          if (\hacklib_cast_as_boolean(
                self::was_uploaded_with_phmm_4_3_6_or_later($url)
              )) {
            $res_path = self::transform_to_watermarked_path($path);
            if (\hacklib_cast_as_boolean(\file_exists($res_path))) {
              \unlink($res_path);
            }
          } else {
            $res_path = self::transform_to_legacy_watermarked_path($path);
            if (\hacklib_cast_as_boolean(\file_exists($res_path))) {
              \unlink($res_path);
            }
          }
        }
      }
    }
  }
}
