<?php
namespace codeneric\phmm\base\frontend {
  require_once ($GLOBALS["HACKLIB_ROOT"]);
  use \codeneric\phmm\base\includes\Project;
  use \codeneric\phmm\base\admin\Settings;
  use \codeneric\phmm\Utils;
  use \codeneric\phmm\enums\AdvancedBoolSettings;
  class CustomPages {
    public static function customize_title($title, $id) {
      $settings = Settings::getCurrentSettings();
      if ($id === $settings[\hacklib_id("gallery_page_id")]) {
        $project_id = Main::get_relevant_id(Shortcodes::GALLERY);
        if ($project_id !== (-1)) {
          $title = \get_the_title($project_id);
        }
      }
      return $title;
    }
    public static function customize_featured_image(
      $thumbnail_id,
      $post_id,
      $meta_key
    ) {
      if ($meta_key !== "_thumbnail_id") {
        return $thumbnail_id;
      }
      $settings = Settings::getCurrentSettings();
      if ($post_id === $settings[\hacklib_id("gallery_page_id")]) {
        $project_id = Main::get_relevant_id(Shortcodes::GALLERY);
        if ($project_id !== (-1)) {
          $tn = Project::get_thumbnail($project_id, false);
          if ($tn !== null) {
            return $tn[\hacklib_id("id")];
          }
        }
      }
      return $thumbnail_id;
    }
    public static function exclude_intelligently_from_wp_list_pages($exclude) {
      return \array_merge($exclude, self::get_page_ids_to_exclude());
    }
    public static function exclude_intelligently_from_nav_menu_items(
      $items,
      $menu,
      $args
    ) {
      $excluded = self::get_page_ids_to_exclude();
      $res = array();
      foreach ($items as $item) {
        $page_id = (int) $item->object_id;
        if (!\hacklib_cast_as_boolean(\in_array($page_id, $excluded))) {
          $res[] = $item;
        }
      }
      return $res;
    }
    private static function get_page_ids_to_exclude() {
      $exclude = array();
      $settings = Settings::getCurrentSettings();
      $portal_page_id = $settings[\hacklib_id("portal_page_id")];
      $client_page_id = $settings[\hacklib_id("client_page_id")];
      $gallery_page_id = $settings[\hacklib_id("gallery_page_id")];
      $ecl = Utils::get_advanced_bool_setting(
        AdvancedBoolSettings::PHMM_EXCLUDE_CLIENT_LOGIN_PAGE
      );
      $emg = Utils::get_advanced_bool_setting(
        AdvancedBoolSettings::PHMM_EXCLUDE_MY_GALLERIES_PAGE
      );
      $eg = Utils::get_advanced_bool_setting(
        AdvancedBoolSettings::PHMM_EXCLUDE_GALLERY_PAGE
      );
      if (\hacklib_cast_as_boolean(\is_user_logged_in())) {
        if (\hacklib_cast_as_boolean($ecl) && ($portal_page_id !== null)) {
          $exclude[] = $portal_page_id;
        }
      } else {
        if (\hacklib_cast_as_boolean($emg) && ($client_page_id !== null)) {
          $exclude[] = $client_page_id;
        }
      }
      if (\hacklib_cast_as_boolean($eg) && ($gallery_page_id !== null)) {
        $exclude[] = $gallery_page_id;
      }
      return $exclude;
    }
    public static function add_display_post_states($post_states, $post) {
      $currentSettings = Settings::getCurrentSettings();
      if ($currentSettings[\hacklib_id("portal_page_id")] === $post->ID) {
        $post_states[\hacklib_id("phmm_page_for_portal")] =
          \__("Client Login Page", "photography-management");
      }
      if ($currentSettings[\hacklib_id("client_page_id")] === $post->ID) {
        $post_states[\hacklib_id("phmm_page_for_client")] =
          \__("My Galleries Page", "photography-management");
      }
      if ($currentSettings[\hacklib_id("gallery_page_id")] === $post->ID) {
        $post_states[\hacklib_id("phmm_page_for_gallery")] =
          \__("Gallery Page", "photography-management");
      }
      if ($currentSettings[\hacklib_id("portfolio_page_id")] === $post->ID) {
        $post_states[\hacklib_id("phmm_page_for_portfolio")] =
          \__("Portfolio Page", "photography-management");
      }
      return $post_states;
    }
  }
}
